<?php

/*
Copyright (C) 2008  Simon Emery

This file is part of OIOpublisher Direct.
*/

if(!defined('oiopub')) die();

//payment class
class oiopub_payment {

	var $log;
	var $data;
	var $res = false;
	
	//init
	function oiopub_payment() {
		if(get_class($this) == 'oiopub_payment' || !is_subclass_of($this, 'oiopub_payment')) {
			die("The oiopub_payment class is abstract, and cannot be called directly.");
		}
	}
	
	//process form
	function process_form($rand_id) {
		return true;
	}
	
	//ipn validate
	function ipn_validate() {
		$this->_post_vars();
		$this->_verify_request();
		$this->_verify_data();
		$this->_update_db();	
	}
	
	//post vars
	function _post_vars() {
		//get post vars
		if(!empty($_POST)) {
			foreach($_POST as $key=>$val) {
				$this->$key = oiopub_clean($val);
			}
		}	
	}
	
	//verify request
	function _verify_request() {
		return true;
	}
	
	//verify data
	function _verify_data() {
		return true;
	}
	
	//update db
	function _update_db() {
		return true;
	}
	
	//purchase data
	function _purchase_data($rand_id) {
		global $oiopub_set, $oiopub_db;
		$rand_id = oiopub_clean($rand_id);
		$this->data = $oiopub_db->GetRow("SELECT * FROM " . $oiopub_set->dbtable_purchases . " WHERE rand_id='$rand_id' LIMIT 1");
	}
	
	//purchase form data
	function _purchase_form_data($rand_id) {
		global $oiopub_set;
		//get purchase data
		$this->_purchase_data($rand_id);
		//access allowed?
		if(empty($this->data) || $this->data->payment_status == 1) {
			header("Location: " . $oiopub_set->plugin_url . "/payment.php?rand=" . $rand_id);
			exit();
		}
		//build purchase options
		$item = oiopub_adtype_info($this->data);
		$description = $item['type'] . " purchase at " . $oiopub_set->site_url;
		$subscription = ($this->data->item_subscription == 1 && $this->data->item_duration > 0) ? 1 : 0;
		$purchases = array( "id"=>$this->data->item_id, "name"=>$item['type'], "description"=>$description, "cost"=>$this->data->payment_amount, "currency"=>$this->data->payment_currency, "quantity"=>1, "subscription"=>$subscription, "duration"=>$this->data->item_duration, "rand_id"=>$this->data->rand_id, "txn_id"=>$this->data->txn_id );
		return $purchases;
	}
	
	//remove subscription
	function _remove_subscription($type='cancel') {
		global $oiopub_set, $oiopub_db;
		//modify subscription value
		$oiopub_db->query("UPDATE " . $oiopub_set->dbtable_purchases . " SET item_subscription='0' WHERE rand_id='" . $this->data->rand_id . "' LIMIT 1");
		//end subscription
		if($type == 'expire') {
			//let cron job handle expiration
			//oiopub_approve("expire", $this->data->item_id);
		}
		//cancel subscription
		if($type == 'cancel') {
			$subject = $oiopub_set->site_name . " - Subscription Cancelled";
			$message = "Message generated by OIOpublisher Direct - a subscription (" . oiopub_type_check($this->data->item_channel) . " purchase, #" . $this->data->item_id . ") has just been cancelled. The database has been automatically updated to remove the subscription, and the purchase will now expire normally.";
			oiopub_mail_client($oiopub_set->admin_mail, $subject, $message);	
		}
	}
	
	//payment success
	function _payment_success($txn_id) {
		global $oiopub_set, $oiopub_db, $oiopub_api, $oiopub_module;
		//update required?
		if($this->data->payment_status == 1) {
			return;
		}
		//get time
		$time_now = time();
		//update purchase
		$oiopub_db->query("UPDATE " . $oiopub_set->dbtable_purchases . " SET payment_txid='$txn_id', payment_time='$time_now', payment_status='1' WHERE rand_id='" . $this->data->rand_id . "' LIMIT 1");
		//update affiliate sale
		if($oiopub_module->affiliates == 1) {
			$oiopub_db->query("UPDATE " . $oiopub_set->dbtable_affiliates_sales . " SET purchase_time='$time_now', purchase_payment='1' WHERE purchase_id='" . $this->data->item_id . "' AND purchase_payment='0' ORDER BY id DESC LIMIT 1");		
		}
		//cache flush
		oiopub_flush_cache();
		//send api call?
		if($this->data->submit_api > 0) {
			$oiopub_api->status_send($this->data->item_id, "payment", 1);
		}
		//send messages
		if($this->data->item_channel == 1 && $this->data->item_status == 1) {
			$subject = $oiopub_set->site_name . " - Paid Review has been paid for!";
			$message = "Message generated by OIOpublisher Direct - a paid review (ID #" . $this->data->item_id . ") has just been paid for. You should now login to the post manager, select the 'unwritten' option from the menu, and write the post for the advertiser.\n\nOnce written, please publish the post, which will automatically notify the advertiser by email.\n\nThanks,\nOIOpublisher";
			oiopub_mail_client($oiopub_set->admin_mail, $subject, $message);
		} elseif($this->data->item_channel != 1 && $this->data->item_status == 3) {
			oiopub_approve("renew", $this->data->item_id);
		} elseif($this->data->item_channel == 4 && $this->data->item_status == 1) {
			$subject = $oiopub_set->site_name . " - Custom Service has been paid for!";
			$message = "Message generated by OIOpublisher Direct - a custom service (ID #" . $this->data->item_id . ") has just been paid for. You should now login to the services manager and complete the custom service requirements for the advertiser.\n\nThanks,\nOIOpublisher";
			oiopub_mail_client($oiopub_set->admin_mail, $subject, $message);
		}	
	}
	
	//payment failed
	function _payment_failed($txn_id) {
		global $oiopub_set, $oiopub_db, $oiopub_api, $oiopub_module;
		//get time
		$time_now = time();
		//update purchase
		$oiopub_db->query("UPDATE " . $oiopub_set->dbtable_purchases . " SET payment_txid='$txn_id', payment_time='$time_now', payment_status='2', payment_log='" . $this->log . "' WHERE rand_id='" . $this->data->rand_id . "' LIMIT 1");
		//update affiliate sale
		if($oiopub_module->affiliates == 1) {
			$oiopub_db->query("UPDATE " . $oiopub_set->dbtable_affiliates_sales . " SET purchase_time='$time_now', purchase_payment='2' WHERE purchase_id='" . $this->data->item_id . "' AND purchase_type='" . $this->data->item_channel . "' LIMIT 1");		
		}
		//flush cache
		oiopub_flush_cache();
		//send api call?
		if($this->data->submit_api > 0) {
			$oiopub_api->status_send($this->data->item_id, "payment", 1);
		}
		//send messages
		if($this->data->item_status != 1) {
			if(empty($txn_id)) $txn_id = "N/A";
			$subject = $oiopub_set->site_name . " - Transaction requires manual review!";
			$message = "Message generated by OIOpublisher Direct - a payment has just been recorded for an advertising purchase at " . $oiopub_set->site_name . ", but it has failed the verification checks. Please review the purchase manually before marking the payment as valid.\n\nThe Transaction ID recorded was " . $txn_id . ", the purchase type was a " . oiopub_type_check($this->data->item_channel) . ", and the purchase ID was " . $this->data->item_id . ".\n\nAs the purchase has yet to be reviewed, the payment status will be adjusted automatically upon approving the purchase.\n\nBelow is a log of the errors recorded\n\n" . $this->log; 
			oiopub_mail_client($oiopub_set->admin_mail, $subject, $message);
		} else {
			if(empty($txn_id)) $txn_id = "N/A";
			$subject = $oiopub_set->site_name . " - Transaction requires manual review!";
			$message = "Message generated by OIOpublisher Direct - a payment has just been recorded for an advertising purchase at " . $oiopub_set->site_name . ", but it has failed the verification checks. Please review the purchase manually before marking the payment as valid.\n\nThe Transaction ID recorded was " . $txn_id . ", the purchase type was a " . oiopub_type_check($this->data->item_channel) . ", and the purchase ID was " . $this->data->item_id . ".\n\nAs the purchase has already been approved, once you have verified the payment was made, please login and use the 'Invalid' dropdown menu from the relevant purchase management panel to mark the payment as valid.\n\nBelow is a log of the errors recorded\n\n" . $this->log;
			oiopub_mail_client($oiopub_set->admin_mail, $subject, $message);
			$subject = $oiopub_set->site_name . " - Transaction requires manual review!";
			$message = "You recently made an approved purchase at " . $oiopub_set->site_name . ". A payment was recorded, but failed the verification checks, and must therefore be checked manually before your purchase will appear on the site.\n\nThanks,\n" . $oiopub_set->site_name;
			oiopub_mail_client($this->data->adv_email, $subject, $message);
		}
	}

}

?>