<?php

/*
Module: Demographics 2.01
Developer: http://www.simonemery.co.uk

Module constructed for OIOpublisher Direct
http://www.oiopublisher.com

Copyright (C) 2008  Simon Emery

This file is part of OIOpublisher Direct.
*/

if(!defined('oiopub')) die();


//module vars
$oio_enabled = isset($oiopub_set->demographics) ? $oiopub_set->demographics['enabled'] : 0;;
$oio_version = "2.01";
$oio_name = __oio("Demographics");
$oio_module = "demographics";
$oio_menu = "";

//min plugin version
$oio_min_version = "2.00";

//check minimum version
oiopub_module_vcheck($oio_module, $oio_min_version);

//democraphics class
class oiopub_demographics {

	//global vars
	var $version;
	
	//init
	function oiopub_demographics($oio_version='') {
		$this->version = $oio_version;
		$this->settings();
		$this->install();
		$this->redirect();
		$this->hooks();
	}
	
	//settings
	function settings() {
		global $oiopub_set;
		//get folder name
		$dir = trim(str_replace('\\', '/', dirname(__FILE__)));
		$exp = explode('/', $dir);
		$folder = $exp[count($exp)-1];
		//misc settings
		$oiopub_set->demographics_folder = $folder;
		$oiopub_set->demographics_dir = $oiopub_set->folder_dir . '/modules/' . $folder;
	}
	
	//install
	function install() {
		global $oiopub_set;
		if(!isset($oiopub_set->demographics) || $oiopub_set->demographics['install'] < $this->version) {
			if(empty($oiopub_set->demographics['install'])) {
				include_once($oiopub_set->modules_dir . '/' . $oiopub_set->demographics_folder . '/install/install.php');
			} else {
				include_once($oiopub_set->modules_dir . '/' . $oiopub_set->demographics_folder . '/install/upgrade.php');
			}
			$oiopub_set->demographics['install'] = $this->version;
			oiopub_update_config('demographics', $oiopub_set->demographics);
		}
	}
	
	//redirect
	function redirect() {
		if(oiopub_is_admin()) {
			if(isset($_GET['page']) && $_GET['page'] == 'oiopub-demographics.php') {
				header("Location: admin.php?page=oiopub-modules.php&module=demographics");
				exit();
			}
		}
	}

	//hooks
	function hooks() {
		global $oiopub_set, $oiopub_hook;
		if(oiopub_is_admin()) {
			if(isset($_GET['page']) && $_GET['page'] == "oiopub-modules.php") {
				if(isset($_GET['module']) && $_GET['module'] == "demographics") {
					$oiopub_hook->add('my_modules', array(&$this, 'admin_options'));
					$oiopub_hook->add('help_desk', array(&$this, 'help'));
				}
			}
		} elseif($oiopub_set->demographics['enabled'] == 1) {
			$oiopub_hook->add('init', array(&$this, 'get_location'));
		}
	}
	
	//get location
	function get_location() {
		global $oiopub_set;
		//admin check
		if(oiopub_is_admin() || $oiopub_set->host_name == "localhost") {
			return;
		}
		//cookie check
		if(isset($_COOKIE['oiopub_location']) && !empty($_COOKIE['oiopub_location'])) {
			$oiopub_set->visitor_country = oiopub_clean($_COOKIE['oiopub_location']);
			return;
		}
		//set vars
		$oiopub_set->visitor_country = "";
		$limit = time() + (24 * 3600);
		//call to web service
		$res = oiopub_file_contents("http://www.geoplugin.net/php.gp?ip=" . $oiopub_set->client_ip);
		//success?
		if(is_string($res) && $res) {
			$geoip = unserialize($res);
			$oiopub_set->visitor_country = $geoip['geoplugin_countryCode'];
		} else {
			$oiopub_set->visitor_country = '';
		}
		//set cookie
		setcookie("oiopub_location", $oiopub_set->visitor_country, $limit, "/");
	}
	
	//help
	function help() {
		global $oiopub_set;
		echo "<b>Section Covered:</b> Demographics Module\n";
		echo "<br /><br />\n";
		echo "This modules enables you to target ads by certain criteria.\n";
		echo "<br /><br />\n";
		echo "<b>Default Ads:</b>\n";
		echo "<br /><br />\n";
		echo "When setting up a default ad, the demographics options will be selectable when this module is enabled. This currently includes location targeting options.\n";
	}
	
	//admin options
	function admin_options() {
		global $oiopub_set;
		if(isset($_POST['oiopub_demographics_enabled'])) {
			$array = array();
			$array['enabled'] = intval($_POST['oiopub_demographics_enabled']);
			oiopub_update_config('demographics', $array);
			unset($array);
		}
		echo "<h2>Ad Demographics</h2>\n";
		echo "This module allows you to target ads based on the visitor viewing the ad.\n";
		echo "<br /><br />\n";
		echo "<form action=\"" . oiopub_clean($_SERVER["REQUEST_URI"]) . "\" method=\"post\">\n";
		echo "<input type='hidden' name='csrf' value='" . oiopub_csrf_token() . "' />\n";
		echo "<b>Enable Demographics?</b>";
		echo "<br /><br />\n";
		echo oiopub_dropmenu_kv($oiopub_set->arr_yesno, "oiopub_demographics_enabled", $oiopub_set->demographics['enabled']);
		echo "&nbsp;&nbsp;<i>do you wish to enable the demographics module?</i>\n";
		echo "<br />\n";
		echo "<div class=\"submit\"><input type=\"submit\" value=\"Update Settings\" /></div>\n";
		echo "</form>\n";
		echo "<br /><br />\n";
		echo "<b>Limitations:</b>\n";
		echo "<br /><br />\n";
		echo "* This module is currently limited to the use of default ads only.\n";
		echo "<br />\n";
		echo "* This module only targets by the visitor's location (country) at this time.\n";
		echo "<br /><br />\n";
		echo "<b>Location Targeting:</b>\n";
		echo "<br /><br />\n";
		if(oiopub_links) {
			echo "* <a href='admin.php?page=oiopub-settings.php&opt=link#defaults'>Text Ads</a>\n";
			echo "<br />\n";
		}
		if(oiopub_banners) {
			echo "* <a href='admin.php?page=oiopub-settings.php&opt=banner#defaults'>Banner Ads</a>\n";
			echo "<br />\n";
		}
		if(oiopub_inline) {
			echo "* <a href='admin.php?page=oiopub-settings.php&opt=inline#defaults'>Inline Ads</a>\n";
			echo "<br />\n";
		}
	}
	
}

//execute class
$oiopub_plugin[$oio_module] = new oiopub_demographics($oio_version);

?>