<?php

/*
Copyright (C) 2008  Simon Emery

This file is part of OIOpublisher Direct.
*/

if(!defined('oiopub')) die();

//paypal class
class oiopub_payment_entrecard extends oiopub_payment {

	//init
	function oiopub_payment_entrecard() {

	}
	
	//process form
	function process_form($rand_id) {
		//get purchase data
		$purchase = $this->_purchase_form_data($rand_id);
		//get params
		$params = $this->_get_params($purchase);
		//form data
		$this->_build_form($params);
	}
	
	//get params
	function _get_params($item) {
		global $oiopub_set;
		//signature params
		$params  = "seller=" . urlencode($oiopub_set->entrecard['seller_id']) . "&";
		$params .= "id=" . urlencode($item['rand_id']) . "&";
		$params .= "description=" . urlencode($item['description']) . "&";
		$params .= "quantity=1&";
		$params .= "cost=" . urlencode(intval($item['cost'])) . "&";
		$params .= "notify=" . urlencode($oiopub_set->entrecard_ipn) . "&";
		$params .= "success=" . urlencode($oiopub_set->entrecard_success . "&rand=" . $item['rand_id']) . "&";
		$params .= "cancel=" . urlencode($oiopub_set->entrecard_failed);
		//sha1 class
		include_once($oiopub_set->folder_dir . '/include/sha1.php');
		//get signature
		$sha1 = new oiopub_sha1;
		$signature = $sha1->output($params . $oiopub_set->entrecard['seller_secret']);
		//return result
		return array($params, $signature);
	}

	//build form
	function _build_form($params) {
		//construct url
		$url = "https://entrecard.com/uapi/1/buy?" . $params[0] . "&signature=" . $params[1];
		//redirect user
		header("Location: " . $url);
		exit();
	}

	//verify request
	function _verify_request() {
		global $oiopub_set;
		//get signature
		$signature = $this->_calc_signature();
		//validate signature (1)
		if(strlen($this->signature) != 40) {
			$this->log .= "An invalid authentication signature was returned by EntreCard\n";
		}
		//validate signature (2)
		if(strcmp($signature, $this->signature) != 0) {
			$this->log .= "The EntreCard API request could not be verified\n";
		}
	}

	//verify data
	function _verify_data() {
		global $oiopub_set;
		//get purchase data
		$this->_purchase_data($this->id);
		//verify transaction
		if(strcmp($this->status, "complete") != 0 && strcmp($this->status, "escrow") != 0) {
			$this->log .= "Transaction marked as " . $this->status . ", not Complete\n";
		}
		if(empty($this->transaction_id)) {
			$this->log .= "No Transaction ID Recorded\n";
		}
		if($this->seller_id != $oiopub_set->entrecard['seller_id']) {
			$this->log .= "EntreCard Seller ID recorded did not match\n";
		}
		if(!$this->data) {
			$this->log .= "No records matched the data sent\n";
		}
		if($this->total != $this->data->payment_amount) {
			$this->log .= "The payment amounts recorded did not match\n";
		}
		if($this->data->payment_currency != "EC") {
			$this->log .= "Incorrect currency used\n";
		}
	}
	
	//update db
	function _update_db() {
		if($this->data && $this->data->payment_status != 1) {
			if(empty($this->log)) {
				//payment success
				$this->_payment_success($this->transaction_id);
			} else {
				//payment failed
				$this->_payment_failed($this->transaction_id);
			}
		}
	}
	
	//calc signature
	function _calc_signature() {
		global $oiopub_set;
		//signature params
		$params  = "status=" . urlencode($this->status) . "&";
		$params .= "seller_id=" . urlencode($this->seller_id) . "&";
		$params .= "total=" . urlencode($this->total) . "&";
		$params .= "id=" . urlencode($this->id) . "&";
		$params .= "transaction_id=" . urlencode($this->transaction_id);
		//sha1 class
		include_once($oiopub_set->folder_dir . '/include/sha1.php');
		//get signature
		$sha1 = new oiopub_sha1;
		$signature = $sha1->output($params . $oiopub_set->entrecard['seller_secret']);
		//return result
		return $signature;
	}

}

?>