<?php

/*
Module: EntreCard Checkout v2.01
Developer: http://www.simonemery.co.uk

Module constructed for OIOpublisher Direct
http://www.oiopublisher.com

Copyright (C) 2008  Simon Emery

This file is part of OIOpublisher Direct.
*/

if(!defined('oiopub')) die();


//module vars
$oio_enabled = 1;
$oio_version = "2.01";
$oio_name = __oio("EntreCard");
$oio_module = "entrecard";
$oio_menu = "payment";

//min plugin version
$oio_min_version = "2.10.b1";

//check minimum version
oiopub_module_vcheck($oio_module, $oio_min_version);


//entrecard class
class oiopub_entrecard {

	//global vars
	var $name;
	var $folder;
	var $version;
	
	//init
	function oiopub_entrecard($version='', $name='') {
		//set vars
		$this->name = $name;
		$this->version = $version;
		//call methods
		$this->settings();
		$this->install();
		$this->redirect();
		$this->hooks();
	}
	
	//settings
	function settings() {
		global $oiopub_set;
		//get folder name
		$dir = trim(str_replace('\\', '/', dirname(__FILE__)));
		$exp = explode('/', $dir);
		$this->folder = $exp[count($exp)-1];
		//misc settings
		$oiopub_set->entrecard_folder = $this->folder;
		$oiopub_set->entrecard_ipn = $oiopub_set->plugin_url . '/modules/' . $this->folder . '/entrecard-ipn.php';
		$oiopub_set->entrecard_form = $oiopub_set->plugin_url . '/modules/' . $this->folder . '/entrecard-form.php';
		$oiopub_set->entrecard_success = $oiopub_set->plugin_url . '/payment.php?do=success';
		$oiopub_set->entrecard_failed = $oiopub_set->plugin_url . '/payment.php?do=failed';
		$oiopub_set->entrecard_subscription = 0;
		//check array
		if(isset($oiopub_set->entrecard)) {
			//set custom currency
			$oiopub_set->entrecard['currency'] = "EC";
			//payment array
			if($oiopub_set->entrecard['enable'] == 1 && $oiopub_set->entrecard['valid'] == 1) {
				$oiopub_set->arr_payment['entrecard'] = $this->name;
			}
		}
	}

	//install
	function install() {
		global $oiopub_set;
		if(!isset($oiopub_set->entrecard) || $oiopub_set->entrecard['install'] < $this->version) {
			if(empty($oiopub_set->entrecard['install'])) {
				include_once($oiopub_set->modules_dir . '/' . $this->folder . '/install/install.php');
			} else {
				include_once($oiopub_set->modules_dir . '/' . $this->folder . '/install/upgrade.php');
			}
			$oiopub_set->entrecard['install'] = $this->version;
			oiopub_update_config('entrecard', $oiopub_set->entrecard);
		}
	}
	
	//uninstall
	function uninstall() {
		global $oiopub_set;
		include_once($oiopub_set->modules_dir . '/' . $this->folder . '/install/uninstall.php');
	}
	
	//page redirect
	function redirect() {
		if(oiopub_is_admin()) {
			if(isset($_GET['page']) && $_GET['page'] == 'oiopub-entrecard.php') {
				header("Location: admin.php?page=oiopub-settings.php&popup=entrecard");
				exit();
			}
		}
	}
	
	//add actions
	function hooks() {
		global $oiopub_hook;
		if(defined('OIOPUB_PURCHASE')) {
			$oiopub_hook->add('pre_purchase', array(&$this, 'pre_purchase'), -1000);
		}
		if(oiopub_is_admin()) {
			if(isset($_GET['page']) && $_GET['page'] == "oiopub-settings.php") {
				$oiopub_hook->add('admin_payment', array(&$this, 'admin_options'));
				if(isset($_GET['popup']) && $_GET['popup'] == "entrecard") {
					$oiopub_hook->add('settings_popup', array(&$this, 'admin_popup'));
				}
			}
			if(isset($_REQUEST['do']) && $_REQUEST['do'] == "oiopub-remove") {
				$oiopub_hook->add('delete_modules', array(&$this, 'uninstall'));
			}
		}
	}
	
	//pre-purchase
	function pre_purchase($item) {
		global $oiopub_set;
		if($item->payment_processor == "entrecard") {
			$item->payment_amount = $item->payment_amount * $oiopub_set->entrecard['credits_ratio'];
		}
	}
	
	//admin options
	function admin_options() {
		global $oiopub_set;
		if($oiopub_set->entrecard['valid'] == 1) {
			if($oiopub_set->entrecard['enable'] == 1) {
				echo "<font color='green'><b>EntreCard Payments Active</b></font> - ";
			} else {
				echo "<font color='blue'><b>EntreCard Payments Disabled</b></font> - ";
			}
		} else {
			echo "<font color='red'><b>EntreCard Setup Incomplete</b></font> - ";
		}
		echo "[<a href='admin.php?page=oiopub-settings.php&popup=entrecard'>edit settings</a>]\n";
		echo "<br /><br />\n";
	}
	
	//admin popup
	function admin_popup() {
		global $oiopub_set;
		$info = ""; $checked = "";
		if(isset($_POST['oiopub_entrecard_seller'])) {
			$enable = intval($_POST['oiopub_entrecard_enable']);
			$seller_id = intval($_POST['oiopub_entrecard_seller']);
			$seller_secret = oiopub_clean($_POST['oiopub_entrecard_secret']);
			$seller_ratio = intval($_POST['oiopub_entrecard_ratio']);
			if($seller_id > 0 && !empty($seller_secret) && $seller_ratio > 0) {
				$oiopub_set->entrecard['enable'] = $enable;
				$oiopub_set->entrecard['valid'] = 1;
				$oiopub_set->entrecard['seller_id'] = $seller_id;
				$oiopub_set->entrecard['seller_secret'] = $seller_secret;
				$oiopub_set->entrecard['credits_ratio'] = $seller_ratio;
				oiopub_update_config('entrecard', $oiopub_set->entrecard);
				$info = "<font color='green'><i><b>Information successfully updated!</b></i></font>";
			} else {
				$oiopub_set->entrecard['valid'] = 0;
				oiopub_update_config('entrecard', $oiopub_set->entrecard);
				$info = "<font color='red'><i><b>Information not validated. Please try again!</b></i></font>";
			}
		}
		if($oiopub_set->entrecard['enable'] == 1) {
			$checked = " checked=\"checked\"";
		}
		echo "<div style='border-top:1px solid #D8D8D8; border-bottom:1px solid #D8D8D8; padding:5px 0 5px 0; margin:5px 0 20px 0;'>\n";
		if(!empty($info)) {
			echo $info . "\n";
		} else {
			echo "<i><b>Integration type:</b> fully automated.</i>\n";
			echo "<br />\n";
			echo "<i><b>Subscriptions Supported:</b> " . ($oiopub_set->entrecard_subscription == 1 ? 'yes' : 'no') . ".</i>\n";		
		}
		echo "</div>\n";
		echo "<form method='post' action='" . $oiopub_set->request_uri . "'>\n";
		echo "<input type='hidden' name='csrf' value='" . oiopub_csrf_token() . "' />\n";
		echo "<b>Enable EntreCard as Payment Option?</b>\n";
		echo "<br /><br />\n";
		echo "<input type=\"checkbox\" name=\"oiopub_entrecard_enable\" value=\"1\"" . $checked . " />\n";
		echo "&nbsp;&nbsp;<i>enabling this will option will let advertisers pay using <a href='http://www.entrecard.com' target='_blank'>entrecard</a></i>\n";
		echo "<br /><br />\n";
		echo "<b>EntreCard Seller ID:</b>\n";
		echo "<br /><br />\n";
		echo "<input type=\"text\" name=\"oiopub_entrecard_seller\" value=\"" . $oiopub_set->entrecard['seller_id'] . "\" size=\"30\" />\n";
		echo "&nbsp;&nbsp;<i>enter your EntreCard <a href='http://entrecard.com/r/api_access' target='_blank'>Seller ID</a></i>\n";
		echo "<br /><br />\n";
		echo "<b>EntreCard Shared Secret:</b>\n";
		echo "<br /><br />\n";
		echo "<input type=\"text\" name=\"oiopub_entrecard_secret\" value=\"" . $oiopub_set->entrecard['seller_secret'] . "\" size=\"30\" />\n";
		echo "&nbsp;&nbsp;<i>the secret string generated in your <a href='http://entrecard.com/r/api_access' target='_blank'>EC account</a></i>\n";
		echo "<br /><br />\n";
		echo "<b>EntreCard Credits Ratio:</b>\n";
		echo "<br /><br />\n";
		echo "<input type=\"text\" name=\"oiopub_entrecard_ratio\" value=\"" . $oiopub_set->entrecard['credits_ratio'] . "\" size=\"30\" />\n";
		echo "&nbsp;&nbsp;<i>how many credits do you want 1 " . $oiopub_set->general_set['currency'] . " to be worth?</i>\n";
		echo "<br /><br />\n";
		echo "<input type='submit' value='Update Settings' />\n";
		echo "</form>\n";
	}
	
	//paypal form
	function form($rand_id) {
		global $oiopub_set;
		echo '<form id="entrecard" action="' . $oiopub_set->entrecard_form . '" method="post">';
		echo '<input type="hidden" name="rand" value="' . $rand_id . '" />';
		echo '<input type="image" src="' . $oiopub_set->plugin_url . '/modules/' . $this->folder . '/images/payec.png" border="0" name="submit" />';
		echo '</form>';
	}
	
}


//execute class
$oiopub_plugin[$oio_module] = new oiopub_entrecard($oio_version, $oio_name);

?>